### Calc_CorrFacs.py. See Readme.txt for overall description of this code.
### c. Harry Desmond (2017), harry.desmond@physics.ox.ac.uk


import numpy as np
import math

'Load halo catalogue, calculate apparent magnitudes & prepare variables'

out_file = open('Correction.dat', 'w')

halos = np.load("Halo_Catalogue.npy")
mvir0 = np.array(halos[:,0])                             # In Msun
rvir0 = np.array(halos[:,1])/(1000.*0.7)                 # In Mpc
conc0 = np.array(halos[:,2])
M0 = np.array(halos[:,3])                                # Absolute magnitude
x0 = np.array(halos[:,4])-200.                           # In Mpc/h
y0 = np.array(halos[:,5])-200.
z0 = np.array(halos[:,6])-200.
Dist0 = np.sqrt(x0*x0 + y0*y0 + z0*z0)/0.7;              # In Mpc

m = M0 + 5.*np.log10(Dist0) + 25.               # Approximate, no dust extinction
maskV = m<12.73
maskD = Dist0<220.
mask = (maskV)&(maskD)

mvir = mvir0[mask]
rvir = rvir0[mask]
conc = conc0[mask]
M = M0[mask]
x = x0[mask]
y = y0[mask]
z = z0[mask]
Dist = Dist0[mask]

Ex = np.loadtxt("Extinction.dat")     # Extinction values for these halos, from Schlegel, Finkbeiner & Davis 1998

m = M + 5.*np.log10(Dist) + 25. + 0.35*Ex - 2.9*70.*Dist/3e5;     # Apparent magnitude
mask = m<12.5;                                                    # 2M++ visibility cut

x1 = x[mask]/0.7                      # Visible and within 220 Mpc
y1 = y[mask]/0.7
z1 = z[mask]/0.7
Dist1 = Dist[mask]

mvir1 = mvir[mask]
rvir1 = rvir[mask]
conc1 = conc[mask]

x20 = x0[maskD]/0.7                   # Within 220 Mpc but not necessarily visible
y20 = y0[maskD]/0.7
z20 = z0[maskD]/0.7

mvir20 = mvir0[maskD]
rvir20 = rvir0[maskD]
conc20 = conc0[maskD]
M20 = M0[maskD]
m20 = M20 + 5.*np.log10(Dist0[maskD]) + 25. + 0.35*np.median(Ex) - 2.9*70.*Dist0[maskD]/3e5

Npart = 1000                    # Specify halo resolution limit
Mcut = Npart * 1.109e8          # Paricle mass in Msun

mask_new = (m20<12.5) | (mvir20>Mcut)

mvir2 = mvir20[mask_new]
rvir2 = rvir20[mask_new]
conc2 = conc20[mask_new]
x2 = x20[mask_new]
y2 = y20[mask_new]
z2 = z20[mask_new]

G = 1.

mask1 = (Dist1<200.)
xM = x1[mask1]
yM = y1[mask1]
zM = z1[mask1]


'Loop over halos to calculate correction factors'

for i in range(len(xM)):
    xt = xM[i]
    yt = yM[i]
    zt = zM[i]
    
    D = np.sqrt((x1-xt)*(x1-xt) + (y1-yt)*(y1-yt) + (z1-zt)*(z1-zt))
    maskD1 = (D<10.)&(D>0.00001)

    count = len(D[maskD1])
    
    if count==0:
        continue
    
    R = np.sqrt((x2 - xt)*(x2 - xt) + (y2 - yt)*(y2 - yt) + (z2 - zt)*(z2 - zt))
    maskR = (R<10.)&(R>0.00001)

    Pa= (G*mvir2[maskR]/R[maskR]) * np.log(1.+conc2[maskR]*R[maskR]/rvir2[maskR])/(np.log(1.+conc2[maskR]) - conc2[maskR]/(1.+conc2[maskR]))
    P = np.sum(Pa)*4.8e-15              # All halos; convert to V/c^2*1e5

    PVa= (G*mvir1[maskD1]/D[maskD1]) * np.log(1.+conc1[maskD1]*D[maskD1]/rvir1[maskD1])/(np.log(1.+conc1[maskD1]) - conc1[maskD1]/(1.+conc1[maskD1]))
    PV = np.sum(PVa)*4.8e-15            # Visible halos only

    ca= G*mvir2[maskR]/R[maskR]**3.
    c = np.sum(ca)

    cVa= G*mvir1[maskD1]/D[maskD1]**3.
    cV = np.sum(cVa)

    acc_x_a = (G * mvir2[maskR] / R[maskR]**3.0) * (np.log(1.+conc2[maskR]*R[maskR]/rvir2[maskR]) - conc2[maskR]*R[maskR]/(rvir2[maskR] + conc2[maskR]*R[maskR]))/(np.log(1.+conc2[maskR])-conc2[maskR]/(1.+conc2[maskR])) * (xt - x2[maskR])
    acc_y_a = (G * mvir2[maskR] / R[maskR]**3.0) * (np.log(1.+conc2[maskR]*R[maskR]/rvir2[maskR]) - conc2[maskR]*R[maskR]/(rvir2[maskR] + conc2[maskR]*R[maskR]))/(np.log(1.+conc2[maskR])-conc2[maskR]/(1.+conc2[maskR])) * (yt - y2[maskR])
    acc_z_a = (G * mvir2[maskR] / R[maskR]**3.0) * (np.log(1.+conc2[maskR]*R[maskR]/rvir2[maskR]) - conc2[maskR]*R[maskR]/(rvir2[maskR] + conc2[maskR]*R[maskR]))/(np.log(1.+conc2[maskR])-conc2[maskR]/(1.+conc2[maskR])) * (zt - z2[maskR])
    acc_x = np.sum(acc_x_a)*3.09e19
    acc_y = np.sum(acc_y_a)*3.09e19
    acc_z = np.sum(acc_z_a)*3.09e19

    acc = np.sqrt(acc_x*acc_x + acc_y*acc_y + acc_z*acc_z)

    accV_x_a = (G * mvir1[maskD1] / D[maskD1]**3.0) * (np.log(1.+conc1[maskD1]*D[maskD1]/rvir1[maskD1]) - conc1[maskD1]*D[maskD1]/(rvir1[maskD1] + conc1[maskD1]*D[maskD1]))/(np.log(1.+conc1[maskD1])-conc1[maskD1]/(1.+conc1[maskD1])) * (xt - x1[maskD1])
    accV_y_a = (G * mvir1[maskD1] / D[maskD1]**3.0) * (np.log(1.+conc1[maskD1]*D[maskD1]/rvir1[maskD1]) - conc1[maskD1]*D[maskD1]/(rvir1[maskD1] + conc1[maskD1]*D[maskD1]))/(np.log(1.+conc1[maskD1])-conc1[maskD1]/(1.+conc1[maskD1])) * (yt - y1[maskD1])
    accV_z_a = (G * mvir1[maskD1] / D[maskD1]**3.0) * (np.log(1.+conc1[maskD1]*D[maskD1]/rvir1[maskD1]) - conc1[maskD1]*D[maskD1]/(rvir1[maskD1] + conc1[maskD1]*D[maskD1]))/(np.log(1.+conc1[maskD1])-conc1[maskD1]/(1.+conc1[maskD1])) * (zt - z1[maskD1])
    accV_x = np.sum(accV_x_a)*3.09e19
    accV_y = np.sum(accV_y_a)*3.09e19
    accV_z = np.sum(accV_z_a)*3.09e19
            
    accV = np.sqrt(accV_x*accV_x + accV_y*accV_y + accV_z*accV_z)
            
    ang = np.arccos((acc_x*accV_x + acc_y*accV_y + acc_z*accV_z)/(acc*accV))        # Angle between acc vector from visible and all objects
            
    Distance = math.sqrt(xt*xt + yt*yt + zt*zt)

    out_file.write( "%f %f %f %f %d %f %f\n" % (P/PV, acc/accV, ang, c/cV, count, Distance, PV) )


out_file.close()
