### Calc_Proxies.py. See Readme.txt for overall description of this code.
### c. Harry Desmond (2017), harry.desmond@physics.ox.ac.uk


import numpy as np
import math
import csv
from random import random

'Load input data'

testpts  = 'TestPoints.dat'                          # Positions to calculate proxies at: RA (deg), DEC (deg), Dist (Mpc)
src = np.genfromtxt('Sources.dat')                   # Source galaxy catalogue
qlm = np.genfromtxt('Smooth_Field.dat')              # Interpolated masses from quasi-linear density modes
correction = np.genfromtxt('Correction.dat')         # c_phi, c_a, c_theta, c_K, N_10, d, phi_vis

outcat = 'Output.dat'                                # Proxies at test points

N = 35
Corr_arr = np.zeros((N,N,N,N,N,N), dtype=np.int)     # c_phi, c_a, c_K, N, D, phi_vis

AllRa = src[:, 0]            # deg
AllDec = src[:, 1]           # deg
AllDis = src[:, 2]           # Mpc
AllMass = src[:, 3]          # Msun
AllRvir = src[:, 4]          # kpc
AllConc = src[:, 5]
            
AllX = qlm[:, 0]             # Mpc
AllY = qlm[:, 1]
AllZ = qlm[:, 2]
AllMass_L = qlm[:, 3]        # Msun

f = csv.writer(open(outcat, 'w'), delimiter=' ')


'Determine the {c,N_10,d,phi_vis} pdf from "correction"'

N = 50
Vcorrmin = 0.999
Vcorrmax = 6.
acorrmin = 0.3
acorrmax = 15.
angcorrmin = 0.
angcorrmax = 2.5
ccorrmin = 0.
ccorrmax = 4.

Vcorrstep = (Vcorrmax - Vcorrmin)/float(N)
acorrstep = (acorrmax - acorrmin)/float(N)
angcorrstep=(angcorrmax - angcorrmin)/float(N)
ccorrstep = (ccorrmax - ccorrmin)/float(N)
    
countmin = 1.
countmax = 520.
distmin = 0.
distmax = 200.
Vmin = -7.8
Vmax = -3.8
    
countstep = (countmax - countmin)/float(N)
diststep = (distmax - distmin)/float(N)
Vstep = (Vmax - Vmin)/float(N)

Vcorr_arr = np.array([[[[0]*N]*N]*N]*N)
acorr_arr = np.array([[[[0]*N]*N]*N]*N)
angcorr_arr = np.array([[[[0]*N]*N]*N]*N)
ccorr_arr = np.array([[[[0]*N]*N]*N]*N)

for i in range(len(correction)):
    el_Vcorr = int(round((correction[i,0] - Vcorrmin)/Vcorrstep))
    el_acorr = int(round((correction[i,1] - acorrmin)/acorrstep))
    el_angcorr = int(round((correction[i,2] - angcorrmin)/angcorrstep))
    el_ccorr = int(round((np.log10(correction[i,3]) - ccorrmin)/ccorrstep))
        
    el_count = int(round((correction[i,4] - countmin)/countstep))
    el_dist = int(round((correction[i,5] - distmin)/diststep))
    el_V = int(round((np.log10(correction[i,6]*1e-5) - Vmin)/Vstep))
        
    if el_count < N and el_count >= 0 and el_dist < N and el_dist >= 0 and el_V < N and el_V >= 0:
        if el_Vcorr >= N:
            el_Vcorr = N-1
        if el_Vcorr < 0:
            el_Vcorr = 0
        if el_acorr >= N:
            el_acorr = N-1
        if el_acorr < 0:
            el_acorr = 0
        if el_ccorr >= N:
            el_ccorr = N-1
        if el_ccorr < 0:
            el_ccorr = 0
        Corr_arr[el_Vcorr][el_acorr][el_ccorr][el_count][el_dist][el_V] += 1
    

'Loop over test points to calculate proxies'
    
G = 4.52e-39                # Msun, kpc, s
for l in open(testpts):
            
    row = l.split(' ')
    ra1, dec1, R1 = float(row[0])*np.pi/180., float(row[1])*np.pi/180., float(row[2])

    # FIRST calculate contribution from halos

    ang = np.arccos(np.cos(dec1) * np.cos(AllDec*np.pi/180.) * np.cos(ra1 - AllRa*np.pi/180.) + np.sin(dec1) * np.sin(AllDec*np.pi/180.))           

    dis = 1000.*np.sqrt(R1**2.0 + AllDis**2.0 - 2.0 * R1 * AllDis * np.cos(ang))

    mask = (dis<10000.) & (dis>0.1)

    pot = - 3.09e16*3.09e16 * (G * AllMass[mask] / dis[mask]) * np.log(1. + AllConc[mask]*dis[mask]/AllRvir[mask]) / (np.log(1.+AllConc[mask]) - AllConc[mask]/(1.+AllConc[mask]))

    curv = math.sqrt(48.)*G*AllMass[mask]/dis[mask]**3.

    element_count = int(round((len(dis[mask]) - countmin)/countstep))
    element_d = int(round((R1-distmin)/diststep))
    if np.sum(pot)==0.:
        element_V=0
    else:
        element_V = int(round((np.log10(-np.sum(pot)*1.11e-11)-Vmin)/Vstep))
    
    if element_count < 0:
        element_count=0
    elif element_count >= N:
        element_count = N-1
    if element_d < 0:
        element_d=0
    elif element_d >= N:
        element_d = N-1
    if element_V < 0:
        element_V=0
    elif element_V >= N:
        element_V = N-1
    
    Corr_3D =Corr_arr[...,element_count,element_d,element_V]

    num = np.sum(Corr_3D)           # Sum over all 3D elements
    
        
    if num==0:
        CorrFac, CorrFac_a, CorrFac_curv = 1., 1., 1.
        continue

    else:
        rand = random()*num

        cum = 0
        for i,j,k in itertools.product(range(N), range(N), range(N)):
            cum += Corr_3D[i][j][k]
            if cum > rand:
                CorrFac = Vcorrmin + (i+0.5)*Vcorrstep
                CorrFac_a = acorrmin + (j+0.5)*acorrstep
                CorrFac_curv1 = ccorrmin + (k+0.5)*ccorrstep
                CorrFac_curv = 10.**CorrFac_curv1
                break
        
    acc_x = 3.09e19 * (G * AllMass[mask] / dis[mask]**3.0) * (np.log(1.+AllConc[mask]*dis[mask]/AllRvir[mask]) - AllConc[mask]*dis[mask]/(AllRvir[mask] + AllConc[mask]*dis[mask]))/(np.log(1.+AllConc[mask])-AllConc[mask]/(1.+AllConc[mask])) * (AllDis[mask]*np.cos(AllDec[mask]*np.pi/180.)*np.cos(AllRa[mask]*np.pi/180.) - R1*np.cos(dec1)*np.cos(ra1))
    acc_y = 3.09e19 * (G * AllMass[mask] / dis[mask]**3.0) * (np.log(1.+AllConc[mask]*dis[mask]/AllRvir[mask]) - AllConc[mask]*dis[mask]/(AllRvir[mask] + AllConc[mask]*dis[mask]))/(np.log(1.+AllConc[mask])-AllConc[mask]/(1.+AllConc[mask])) * (AllDis[mask]*np.cos(AllDec[mask]*np.pi/180.)*np.sin(AllRa[mask]*np.pi/180.) - R1*np.cos(dec1)*np.sin(ra1))
    acc_z = 3.09e19 * (G * AllMass[mask] / dis[mask]**3.0) * (np.log(1.+AllConc[mask]*dis[mask]/AllRvir[mask]) - AllConc[mask]*dis[mask]/(AllRvir[mask] + AllConc[mask]*dis[mask]))/(np.log(1.+AllConc[mask])-AllConc[mask]/(1.+AllConc[mask])) * (AllDis[mask]*np.sin(AllDec[mask]*np.pi/180.) - R1*np.sin(dec1))

    V_out = np.sum(pot)*CorrFac
    a1 = np.sum(acc_x)*CorrFac_a
    a2 = np.sum(acc_y)*CorrFac_a
    a3 = np.sum(acc_z)*CorrFac_a

    curv_tot = np.sum(curv)*CorrFac_curv

    # SECOND calculate contribution from long-wavelength modes

    x1 = R1*np.cos(dec1)*np.cos(ra1)        # Cartesian positions of test objects
    y1 = R1*np.cos(dec1)*np.sin(ra1)
    z1 = R1*np.sin(dec1)

    disL = 1000.*np.sqrt((AllX - x1)*(AllX - x1) + (AllY - y1)*(AllY - y1) + (AllZ - z1)*(AllZ - z1))

    maskL = (disL<10000.) & (disL>0.1)

    potL = - 3.09e16*3.09e16 * G * AllMass_L[maskL] / disL[maskL]
    curvL = math.sqrt(48.)*G*AllMass_L[maskL]/disL[maskL]**3.

    acc_xL = 3.09e19 * (G * AllMass_L[maskL] / disL[maskL]**3.0) * (AllX[maskL] - x1)
    acc_yL = 3.09e19 * (G * AllMass_L[maskL] / disL[maskL]**3.0) * (AllY[maskL] - y1)
    acc_zL = 3.09e19 * (G * AllMass_L[maskL] / disL[maskL]**3.0) * (AllZ[maskL] - z1)

    V_outL = np.sum(potL)
    a1L = np.sum(acc_xL)
    a2L = np.sum(acc_yL)
    a3L = np.sum(acc_zL)
    aL = np.sqrt(a1L*a1L + a2L*a2L + a3L*a3L)
    curvL = np.sum(curvL)

    # THIRD sum and output

    V_out_TOT = V_out + V_outL
    curv_TOT = curv_tot + curvL
    a1_TOT = a1+a1L
    a2_TOT = a2+a2L
    a3_TOT = a3+a3L
    a_out_TOT = math.sqrt(a1_TOT*a1_TOT + a2_TOT*a2_TOT + a3_TOT*a3_TOT)

    f.writerow([ra1*180./np.pi, dec1*180./np.pi, R1, V_out_TOT*1.11e-11, a_out_TOT, curv_TOT/9.e20])
    # RA (deg), DEC (deg), Distance (Mpc), \Phi/c^2, |\vec{a}| (km s^{-2}), K (cm^{-2})
